#include "koala.hh"


// Deliver external events. The (events) bitset is consulted synchronously at
// appropriate places in the fetch-execute loop.

void
Koala::deliver_cold_reset()
{
    if (trace_level >= report_interrupts)
	log("[%lx] Cold reset received", pc);
    events |= cold_reset_event;
}

void
Koala::deliver_soft_reset()
{
    if (trace_level >= report_interrupts)
	log("[%lx] Soft reset received", pc);
    events |= soft_reset_event;
}

void
Koala::deliver_nmi()
{
    if (trace_level >= report_interrupts)
	log("[%lx] NMI reset received", pc);
    events |= nmi_event;
}

void
Koala::deliver_interrupt(unsigned int n)
{
    if (n < 0 || n > 5)
	log("[%lx] Invalid interrupt #%d received.", pc, n);
    else {
	if (trace_level >= report_interrupts)
	    log("[%lx] Interrupt #%d received", pc, n);
	events |= 1 << (n + Cause_IP2);
    }
}

void
Koala::clear_interrupt(unsigned int n)
{
    if (n < 0 || n > 5)
	log("[%lx] Invalid interrupt #%d cleared.", pc, n);
    else {
	events &= ~(1 << (n + Cause_IP2));
    }
}
