#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "module.hh"
#include "sulima.hh"

static char *console_dev;
static Console *console;

static struct termios orig_termios;
static bool restore_termios = false;
static const int poll_interval = 0x10000;

// Initialize the console subsystem.

void
Sulima::create_console_subsystem()
{
    // Link related variables. 
    define("console", console_dev);
}

void
Sulima::enable_console(CPU *cpu)
{
    struct termios termios;

    console = dynamic_cast<Console *>(find_module(console_dev));
    if (!console) {
	throw Error("\"%#s\" is not a Console module.", console_dev);
    }

    io_fd = STDIN_FILENO;
    if (tcgetattr(io_fd, &orig_termios) != -1) {
	termios = orig_termios;
	termios.c_iflag &= ~(INLCR|ICRNL);
	termios.c_lflag &= ~(ICANON|ECHO);
	if (tcsetattr(io_fd, TCSANOW, &termios) != -1)
	    restore_termios = true;
    }

    ioqueue->insert(this);
}

// Destroy the subsystem.

void
Sulima::destroy_console_subsystem()
{
    ioqueue->remove(this);
    if (restore_termios)
	tcsetattr(STDIN_FILENO, TCSANOW, &orig_termios);
}

void
Sulima::io_poll()
{
    unsigned char c;
    int len;

    len = read(io_fd, &c, 1);
    if ((len > 0) && (console != NULL))
	console->input((int)c);
}

