#include <checkpoint.hh>
#include <cpu.hh>
#include <inttypes.hh>


// An event representing external MIPS interrupts.

SerialType<CPU::InterruptEvent> CPU::InterruptEvent::type(
    "CPU::InterruptEvent"
);


CPU::InterruptEvent::InterruptEvent(Checkpoint& cp)
    : Serializable(cp), Event(cp)
{
    SerialID cid = extract<SerialID>(cp);
    cpu = dynamic_cast<CPU *>(find_object(cid));
    if (!cpu)
	throw Checkpoint::SyntaxError(cp, "%d is not a valid CPU ID",
				      UInt32(cid));
    num = extract<Int16>(cp);
}

void
CPU::InterruptEvent::invoke()
{
    cpu->deliver_interrupt(num);
}

void
CPU::InterruptEvent::checkpoint(Checkpoint& cp, bool parent) const
{
    if (parent)
	cp << type << ' ' << id << '\n';
    Event::checkpoint(cp);
    cp << cpu->id << ' ' << num << '\n';
}
